/*
 * File   : lps.c
 * Purpose: This file contains the management for manipulating records.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998, 1999 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/Chooser.h>
#include <Xmt/InputField.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Dialogs.h>

#include <Xbae/Matrix.h>
#include "platten.h"
#include "cdrom.h"

EXEC SQL include sqlca;
EXEC SQL include sqlda;
EXEC SQL include sqltypes;
EXEC SQL include datetime;

#define lpsTITEL	0
#define lpsSINGER	1
#define lpsNUMBER	2
#define lpsSONG		3

/*
 * Strukturen zum austauschen von Daten in Dialogboxen.
 */
typedef struct
{
	char lpSearch[41];
} lpmask_t;

typedef struct
{
	long lpNum;
	char lpTitel[61];
	int  lpKenn;
	char lpDate[11];
	int  lpCategory;
	char lpDiscid[9];
} lplabel_t;

typedef struct
{
	int repeat;
	long snum;
	int spos;
	char stitel[41];
	char ssinger[41];
	char szeit[6];
} lpdchild_t;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct
{
	long lpnum;
	char lptitel[61];
	char lpkenn[3];
	long lpdate;
	char lpCategory[21];
	long lpDiscid;
} lpmaster_t;

typedef struct
{
	long snum;
	int spos;
	char stitel[41];
	char ssinger[41];
	char szeit[6];
} lpchild_t;

lpmaster_t lpmaster;
lpchild_t lpchild;
EXEC SQL END DECLARE SECTION;

XtResource lpmask_resource[] = {
	{ "lpSearch", "lpSearch", XmtRBuffer, XmtSizeOf (lpmask_t, lpSearch),
	  XtOffsetOf (lpmask_t, lpSearch), XtRImmediate, NULL}
};

XtResource lplabel_resource[] = {
	{ "lpNum", "lpNum", XtRInt, sizeof (long),
	  XtOffsetOf (lplabel_t, lpNum), XtRImmediate, NULL},
	{ "lpTitel", "lpTitel", XmtRBuffer, XmtSizeOf (lplabel_t, lpTitel),
	  XtOffsetOf (lplabel_t, lpTitel), XtRImmediate, NULL},
	{ "lpKenn", "lpKenn", XtRInt, sizeof (int),
	  XtOffsetOf (lplabel_t, lpKenn), XtRImmediate, NULL},
	{ "lpDate", "lpDate", XmtRBuffer, XmtSizeOf (lplabel_t, lpDate),
	  XtOffsetOf (lplabel_t, lpDate), XtRImmediate, NULL},
	{ "lpCategory", "lpCategory", XtRInt, sizeof (int),
	  XtOffsetOf (lplabel_t, lpCategory), XtRImmediate, NULL},
	{ "lpDiscid", "lpDiscid", XmtRBuffer, XmtSizeOf (lplabel_t, lpDiscid),
	  XtOffsetOf (lplabel_t, lpDiscid), XtRImmediate, NULL}
};

XtResource lpchild_resource[] = {
	{ "sNum", "sNum", XtRInt, sizeof (long),
	  XtOffsetOf (lpchild_t, snum), XtRImmediate, NULL},
	{ "sPos", "sPos", XtRInt, sizeof (int),
	  XtOffsetOf (lpchild_t, spos), XtRImmediate, NULL},
	{ "sTitel", "sTitel", XmtRBuffer, XmtSizeOf (lpchild_t, stitel),
	  XtOffsetOf (lpchild_t, stitel), XtRImmediate, NULL},
	{ "sSinger", "sSinger", XmtRBuffer, XmtSizeOf (lpchild_t, ssinger),
	  XtOffsetOf (lpchild_t, ssinger), XtRImmediate, NULL},
	{ "sZeit", "sZeit", XmtRBuffer, XmtSizeOf (lpchild_t, szeit),
	  XtOffsetOf (lpchild_t, szeit), XtRImmediate, NULL}
};

XtResource lpdchild_resource[] = {
	{ "chRepeat", "chRepeat", XtRInt, sizeof (int),
	  XtOffsetOf (lpdchild_t, repeat), XtRImmediate, NULL},
	{ "sNum", "sNum", XtRInt, sizeof (long),
	  XtOffsetOf (lpdchild_t, snum), XtRImmediate, NULL},
	{ "sPos", "sPos", XtRInt, sizeof (int),
	  XtOffsetOf (lpdchild_t, spos), XtRImmediate, NULL},
	{ "sTitel", "sTitel", XmtRBuffer, XmtSizeOf (lpdchild_t, stitel),
	  XtOffsetOf (lpdchild_t, stitel), XtRImmediate, NULL},
	{ "sSinger", "sSinger", XmtRBuffer, XmtSizeOf (lpdchild_t, ssinger),
	  XtOffsetOf (lpdchild_t, ssinger), XtRImmediate, NULL},
	{ "sZeit", "sZeit", XmtRBuffer, XmtSizeOf (lpdchild_t, szeit),
	  XtOffsetOf (lpdchild_t, szeit), XtRImmediate, NULL}
};

/*
 * Globale Variablen fr diese Datei.
 */
Widget rclWidget = (Widget)0;
Widget UpdrclWidget = (Widget)0;
Widget qLpWidget = (Widget)0;
Widget lpKennWidget = (Widget)0;
Widget lpCategoryWidget = (Widget)0;
Widget rccWidget = (Widget)0;
Widget UpdrccWidget = (Widget)0;

char stmaster[81];
char stchild[128];
char Singer[41];
int posCount;
int listMasterPos = 0;
Boolean fromLps; 
Boolean bDialogReady = False;
int lpsSearch = 0;
lpdchild_t lpdchild;

/*
 * Funktionsprototypen die nur in dieser Datei verwendet werden.
 */
void UpdateLpLabel (Widget list);
void UpdateLpCuts (Widget list);
void lpsFillChild (char *str);
void lpsListLp (Widget w, long num);
 
/*
 * Handling der Dialogbox zur Erfassung neuer Tontrgerlabel.
 */
void NewLP ()
{
lplabel_t lplabel;

	XmtDisplayBusyCursor (toplevel);

	if (!rclWidget)
	   rclWidget = XmtBuildQueryDialog (toplevel, "rcl", 
		lplabel_resource, XtNumber (lplabel_resource), NULL);

	memset ((void *)&lplabel, 0, sizeof (lplabel_t));

	XmtDialogDoSync (rclWidget, &lplabel);
	XmtDisplayDefaultCursor (toplevel);
}

void AddLpLabelCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
	char lpKenn[3];
	String lpCategory;
EXEC SQL END DECLARE SECTION;
lplabel_t lplabel;
int item, i;
char hv0[128];
 
	XmtDisplayBusyCursor (rclWidget);
	XtSetSensitive (XmtNameToWidget (toplevel, "*rcl*lpDiscid"), False);
	XmtDialogGetDialogValues (rclWidget, &lplabel);
	lpmaster.lpnum = 0;
	strcpy (lpmaster.lptitel, lplabel.lpTitel);
	XtVaGetValues (lpKennWidget, XmtNstate, &item, NULL);

	switch (item)
	{
	   case 0: strcpy (lpKenn, "LP"); break;
	   case 1: strcpy (lpKenn, "CD"); break;
	   case 2: strcpy (lpKenn, "CM"); break;
	   case 3: strcpy (lpKenn, "MC"); break;
	   case 4: strcpy (lpKenn, "MD"); break;
	}

	if ((lpmaster.lpdate = CharToIDate (rclWidget, lplabel.lpDate)) == 0L)
	{
	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	lpCategory = *(String *)XmtChooserGetValue (
		XmtNameToWidget (toplevel, "*rcl*lpCategory"));

	if (item == 1)
	   lpmaster.lpDiscid = strtol (lplabel.lpDiscid, NULL, 16);
	else
	   lpmaster.lpDiscid = 0L;

	if (has_trans)
	   EXEC SQL begin work;

	EXEC SQL insert into lpmaster values (0, :lpmaster.lptitel,
		 :lpKenn, :lpmaster.lpdate, :lpmaster.lpDiscid, :lpCategory);

	lplabel.lpNum = sqlca.sqlerrd[1];

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (rclWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	if (has_trans)
	   EXEC SQL commit work;

	XmtDialogSetDialogValues (rclWidget, &lplabel);
	XmtDisplayDefaultCursor (rclWidget);
}

/*
 * Folgende Funktion wird aufgerufen, wenn eine DiscID eingegeben wurde,
 * aber alle anderen Eingabezeilen noch leer sind. In diesem Fall wird
 * nach einem CDDB Eintrag gesucht und, sofern ein passender gefunden
 * wird, die Daten aus diesem bernommen. Dadurch brauchen CDs nicht
 * doppelt erfasst werden.
 */
void GetCDDBCallback (Widget w, XtPointer tag, XtPointer data)
{
int fd;
lplabel_t lplabel;
char hv0[256], hv1[256], *p;
String Category;

	if (!lpCategoryWidget)
	   return;

	XmtDialogGetDialogValues (rclWidget, &lplabel);

	if (strlen (lplabel.lpTitel))
	   return;

	if (!strlen (lplabel.lpDiscid))
	   return;

	XmtDisplayBusyCursor (rclWidget);
	Category = *(String *)XmtChooserGetValue (lpCategoryWidget);
	sprintf (hv0, "%s/%s/%s", app_resources.CDDBPath, Category, lplabel.lpDiscid);

	if ((fd = open (hv0, O_RDONLY, 0400)) == -1)
	{
	   XmtDisplayError (w, "ErrorNoCDDB", NULL);
	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	while (ReadLine (fd, &hv1[0], sizeof (hv1)) > 0)
	{
	   if (hv1[0] == '#')
	      continue;

	   if ((p = strchr (hv1, ':')) == NULL) 
	      if ((p = strchr (hv1, '=')) == NULL)
	         continue;

	   *p = 0;
	   p++;

	   if (!strcmp (hv1, "DTITLE"))
	   {
	      strncpy (lplabel.lpTitel, ltrim (rtrim (p)), sizeof (lplabel.lpTitel));
	      break;
	   }
	}

	close (fd);

	if (!strlen (lplabel.lpTitel))
	{
	   XmtDisplayError (w, "ErrorWrongCDDB", NULL);
	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	XmtDialogSetDialogValues (rclWidget, &lplabel);
	XmtDisplayDefaultCursor (rclWidget);
}

/*
 * Folgende Funktion wird aufgerufen, wenn der Button 'Auto' aus dem
 * Dialog 'Neues LP-Label erfassen', aufgerufen wird. Die Funktion
 * verlangt das die korrekte AudioCD eingelegt wird. Danach wird die
 * DiscID ermittelt (libcdaudio!!) und im CDDB-Database der
 * entsprechnde Eintrag gesucht. Wird selbiger gefunden, werden alle
 * verfuegbaren DAten ausgelesen. Damit wird zum einen der
 * Plattentitel eingetragen und die einzelnen Cuts der CD.
 */
void rclAutoCallback ()
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
	long anzahl;
	long discid;
	char lpKenn[3], *lpCategory;
EXEC SQL END DECLARE SECTION;
int i;
lplabel_t lplabel;
char hv0[256], hv1[256], hv2[128], *p;
String Category;
cddb_title_t cddb;
Boolean bret;

	if (!lpCategoryWidget)
	   return;

	XmtDialogGetDialogValues (rclWidget, &lplabel);

	if (strlen (lplabel.lpTitel))
	   return;

	bret = XmtDisplayWarningAndAsk (rclWidget, "LpsInsertCD",
			"Please insert an audio CD-ROM into your drive",
			"Ok", "Cancel", XmtNoButton, NULL);

	if (!bret)
	   return;

	XmtDisplayBusyCursor (rclWidget);
	Category = *(String *)XmtChooserGetValue (lpCategoryWidget);

	if (cddbGetDiscInfo (&cddb, 0L, app_resources.cdrom, 
		app_resources.CDDBPath, Category))
	{
	   XmtDisplayError (rclWidget, "ErrorNoCDDB", NULL);
	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	/*
	 * Da das Auslesen der CD funktioniert hat, koennen wir nun
	 * die Daten in die Datenbank eintragen. Damit eine CD nicht
	 * doppelt erfasst wird, suchen wir zuerst nach der DiscID.
	 * Finden wir diese, wird kein Eintrag gemacht.
	 */
	discid = cddb.discid;
	EXEC SQL select count(*) into :anzahl from lpmaster
			where lpmaster.lpdiscid = :discid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   fprintf (stderr, "platten: Error: select count(*) into :anzahl from lpmaster where lpmaster.lpdiscid = %ld\n", discid);
	   DisplaySqlError (rclWidget);
	   return;
	}

	if (anzahl != 0L)
	{
	   XmtDisplayError (rclWidget, "ErrorIDExist", NULL);
	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}
	   
	/*
	 * Da die DiscID der CD nicht gefunden wurde, ist sie nicht
	 * vorhanden oder ohne DiscID erfasst worden. Egal. Als erstes
	 * muessen wir den Mastersatz in die Datenbank einfuegen um
	 * eine Plattennummer zu erhalten. Dann koennen wir die Cuts
	 * der CD in die Child table eintragen.
	 */
	strncpy (lpmaster.lptitel, cddb.title, sizeof (lpmaster.lptitel));
	strcpy (lpKenn, "CD");
	lpmaster.lpdate = 0L;
	lpCategory = Category;

	if (has_trans)
	   EXEC SQL begin work;

	EXEC SQL insert into lpmaster values (0, :lpmaster.lptitel,
		 :lpKenn, NULL, :discid, :lpCategory);

	lpmaster.lpnum = sqlca.sqlerrd[1];

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   fprintf (stderr, "platten: Error: insert into lpmaster values (0, %s, %s, NULL, %ld, %s\n", lpmaster.lptitel, lpKenn, discid, lpCategory);
	   DisplaySqlError (rclWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (rclWidget);
	   return;
	}

	for (i = 0; i < cddb.tracks; i++)
	{
	   lpchild.snum = lpmaster.lpnum;
	   lpchild.spos = i + 1;
	   strncpy (lpchild.stitel, cddb.entry[i].song, sizeof (lpchild.stitel));

	   if (cddb.entry[i].singer[0] == '\0')
	      strcpy (lpchild.ssinger, "-");
	   else
	      strncpy (lpchild.ssinger, cddb.entry[i].singer, sizeof (lpchild.ssinger));
	   strcpy (lpchild.szeit, cddb.entry[i].zeit);

	   EXEC SQL insert into lpchild values (:lpchild.snum,
		 :lpchild.spos, :lpchild.stitel, :lpchild.ssinger,
		 :lpchild.szeit);

	   if (SQLCODE != 0 || CheckSqlError ())
	   {
	      fprintf (stderr, "platten: Error: insert into lpchild values (%ld, %d, %s, %s, %s\n",
			lpchild.snum, lpchild.spos, lpchild.stitel,
			lpchild.ssinger, lpchild.szeit);
	      DisplaySqlError (rclWidget);

	      if (has_trans)
	         EXEC SQL rollback work;

	      XmtDisplayDefaultCursor (rclWidget);
	      return;
	   }
	}

	if (has_trans)
	   EXEC SQL commit work;

	XmtDisplayDefaultCursor (rclWidget);

	/*
	 * Bevor wir das Fenster wieder schliessen, listen wir die
	 * neue CD in den beiden Listboxen auf.
	 */
	lpsListLp (rclWidget, lpmaster.lpnum);

	/*
	 * Alles fertig! Nun nur noch das Fenster schliessen.
	 */
	XmtDialogCancelCallback ();
}

/*
 * Folgende Funktion kann nur im Fenster "lps" durch einen der Buttons
 * aufgerufen werden. Die Funktion entscheidet aufgrund der selektierten
 * Zeilen in den beiden Listboxen des Dialogfensters, was Upgedated werden
 * soll und ruft die entsprechende Funktion auf, welche wiederum ein
 * Dialogfenster fuer das Update anzeigt.
 */
void UpdateLpCallback (Widget w, XtPointer tag, XtPointer data)
{
Widget mlist = XmtNameToWidget (toplevel, "*lps*lpMaster");
Widget clist = XmtNameToWidget (toplevel, "*lps*lpCuts");
int msel, csel, col;
Boolean aret, answer;

	XbaeMatrixFirstSelectedCell (mlist, &msel, &col);
	XbaeMatrixFirstSelectedCell (clist, &csel, &col);

	if (msel < 0 && csel < 0)
	{
	   XmtDisplayWarning (w, "ErrorSelection", NULL);
	   return;
	}

	if (msel >= 0 && csel >= 0)
	{
	   aret = XmtAskForBoolean (w, "LpsUpdateWhat", NULL, NULL, NULL,
			NULL, XmtCancelButton, XmDIALOG_QUESTION, True,
			&answer, NULL);

	   if (!aret)
	      return;

	   if (answer)		/* True = Update lpmaster */
	      UpdateLpLabel (mlist);
	   else
	      UpdateLpCuts (clist);

	   return;
	}

	if (msel >= 0)
	   UpdateLpLabel (mlist);

	if (csel >= 0)
	   UpdateLpCuts (clist);
}

/*
 * Folgende Funktion kann nur aus der Funktion "UpdateLpCallback"
 * aufgerufen werden. Die Funktion holt den selektierten Datensatz
 * aus der Datenbank und uebergibt die Daten einer Dialogbox.
 * Wenn Der User seine Aenderungen getaetigt hat, werden die Aenderungen
 * in die Datenbank eingetragen.
 */
void UpdateLpLabel (Widget list)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
	char lpKenn[3], *lpCategory;
	long inum;
EXEC SQL END DECLARE SECTION;
lplabel_t lplabel;
int i, x, y;
char num[11], hv0[128];
String sCell;
 
	XmtDisplayBusyCursor (qLpWidget);
	/*
	 * Als erstes muessen wir den zu ndernden Datensatz aus
	 * der Datenbank holen und in die Dialogbox eintragen.
	 */
	XbaeMatrixFirstSelectedCell (list, &x, &y);
	sCell = XbaeMatrixGetCell (list, x, 0);
	inum = atol (sCell);

	if (inum <= 0L)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	EXEC SQL select * into :lpmaster.lpnum,
			:lpmaster.lptitel, :lpmaster.lpkenn,
			:lpmaster.lpdate, :lpmaster.lpDiscid,
			:lpmaster.lpCategory from lpmaster where
			lpmaster.lpnum = :inum;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	rtrim (lpmaster.lptitel);
	lplabel.lpNum = lpmaster.lpnum;
	strcpy (lplabel.lpTitel, lpmaster.lptitel);

	if (!strcmp (lpmaster.lpkenn, "LP"))
	   lplabel.lpKenn = 0;
	else if (!strcmp (lpmaster.lpkenn, "CD"))
	   lplabel.lpKenn = 1;
	else if (!strcmp (lpmaster.lpkenn, "CM"))
	   lplabel.lpKenn = 2;
	else if (!strcmp (lpmaster.lpkenn, "MC"))
	   lplabel.lpKenn = 3;
	else if (!strcmp (lpmaster.lpkenn, "MD"))
	   lplabel.lpKenn = 4;
	else
	   lplabel.lpKenn = 0;

	if (lpmaster.lpdate < 1000L)
	   lplabel.lpDate[0] = 0;
	else
	   rfmtdate (lpmaster.lpdate, app_resources.datePattern, &lplabel.lpDate[0]);

	rtrim (lpmaster.lpCategory);

	if (!strcmp (lpmaster.lpCategory, "rock"))
	   lplabel.lpCategory = 0;
	else if (!strcmp (lpmaster.lpCategory, "classical"))
	   lplabel.lpCategory = 1;
	else if (!strcmp (lpmaster.lpCategory, "country"))
	   lplabel.lpCategory = 2;
	else if (!strcmp (lpmaster.lpCategory, "data"))
	   lplabel.lpCategory = 3;
	else if (!strcmp (lpmaster.lpCategory, "folk"))
	   lplabel.lpCategory = 4;
	else if (!strcmp (lpmaster.lpCategory, "jazz"))
	   lplabel.lpCategory = 5;
	else if (!strcmp (lpmaster.lpCategory, "misc"))
	   lplabel.lpCategory = 6;
	else if (!strcmp (lpmaster.lpCategory, "newage"))
	   lplabel.lpCategory = 7;
	else if (!strcmp (lpmaster.lpCategory, "reggae"))
	   lplabel.lpCategory = 8;
	else if (!strcmp (lpmaster.lpCategory, "blues"))
	   lplabel.lpCategory = 9;
	else if (!strcmp (lpmaster.lpCategory, "soundtrack"))
	   lplabel.lpCategory = 10;
	else
	   lplabel.lpCategory = 0;

	if (lplabel.lpKenn == 1)
	   sprintf (lplabel.lpDiscid, "%08x", lpmaster.lpDiscid);
	else
	   lplabel.lpDiscid[0] = 0;

	if (!UpdrclWidget)
	   UpdrclWidget = XmtBuildQueryDialog (toplevel, "Updrcl", 
		lplabel_resource, XtNumber (lplabel_resource), NULL);
	else
	   XtSetSensitive (XmtNameToWidget (toplevel, "*Updrcl*lpDiscid"),
		(lplabel.lpKenn == 1) ? True : False);

	if (!XmtDialogDoSync (UpdrclWidget, &lplabel))
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	/*
	 * Nach dem der User hier etwas geaendert hat (oder auch nicht)
	 * muss es in die Datenbank eingetragen werden.
	 */
	strcpy (lpmaster.lptitel, lplabel.lpTitel);

	switch (lplabel.lpKenn)
	{
	   case 0: strcpy (lpmaster.lpkenn, "LP"); break;
	   case 1: strcpy (lpmaster.lpkenn, "CD"); break;
	   case 2: strcpy (lpmaster.lpkenn, "CM"); break;
	   case 3: strcpy (lpmaster.lpkenn, "MC"); break;
	   case 4: strcpy (lpmaster.lpkenn, "MD"); break;
	   default: strcpy (lpmaster.lpkenn, "LP"); break;
	}

	if ((lpmaster.lpdate = CharToIDate (qLpWidget, lplabel.lpDate)) == 0L)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	switch (lplabel.lpCategory)
	{
	   case  0: strcpy (lpmaster.lpCategory, "rock"); break;
	   case  1: strcpy (lpmaster.lpCategory, "classical"); break;
	   case  2: strcpy (lpmaster.lpCategory, "country"); break;
	   case  3: strcpy (lpmaster.lpCategory, "data"); break;
	   case  4: strcpy (lpmaster.lpCategory, "folk"); break;
	   case  5: strcpy (lpmaster.lpCategory, "jazz"); break;
	   case  6: strcpy (lpmaster.lpCategory, "misc"); break;
	   case  7: strcpy (lpmaster.lpCategory, "newage"); break;
	   case  8: strcpy (lpmaster.lpCategory, "reggae"); break;
	   case  9: strcpy (lpmaster.lpCategory, "blues"); break;
	   case 10: strcpy (lpmaster.lpCategory, "soundtrack"); break;
	   default: strcpy (lpmaster.lpCategory, "rock");
	}

	if (lplabel.lpKenn == 1)
	   lpmaster.lpDiscid = strtoul (lplabel.lpDiscid, NULL, 16);
	else
	   lpmaster.lpDiscid = 0L;

	if (has_trans)
	   EXEC SQL begin work;

	EXEC SQL update lpmaster set lptitel = :lpmaster.lptitel,
		 lpkenn = :lpmaster.lpkenn, lpdate = :lpmaster.lpdate,
		 lpdiscid = :lpmaster.lpDiscid,
		 lpcategory = :lpmaster.lpCategory
		 where lpnum = :lpmaster.lpnum;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (has_trans)
	   EXEC SQL commit work;

	/*
	 * Um die Aenderung sichtbar zu machen, aendern wir noch die
	 * entsprechende Zeile in der Listbox.
	 */
	sprintf (stmaster, "%ld", lpmaster.lpnum);
	XbaeMatrixSetCell (list, x, 0, stmaster);
	XbaeMatrixSetCell (list, x, 1, lpmaster.lptitel);
	XbaeMatrixSetCell (list, x, 2, lpmaster.lpkenn);
	XbaeMatrixSetCell (list, x, 3, lplabel.lpDate);
	XmtDisplayDefaultCursor (qLpWidget);
}

void lpsEnterCellCallback (Widget w, XtPointer tag, XtPointer data)
{
XbaeMatrixEnterCellCallbackStruct *ec = (XbaeMatrixEnterCellCallbackStruct *)data;

	ec->doit = False;
	XbaeMatrixDeselectAll (w);
	XbaeMatrixSelectRow (w, ec->row);
	XbaeMatrixUnhighlightCell (w, ec->row, ec->column);
}

void rclClearValuesCallback (Widget w, XtPointer tag, XtPointer data)
{
lplabel_t lplabel;

	memset ((void *)&lplabel, 0, sizeof (lplabel_t));
	XmtDialogSetDialogValues (rclWidget, &lplabel);
}

void rclInitChooser (Widget wk, Widget wd)
{
	lpKennWidget = wk;
	lpCategoryWidget = wd;
}

void UpdrclInitDiscid (Widget w)
{
int item = 0;

	if (!strcmp (lpmaster.lpkenn, "CD"))
	   item = 1;

	XtSetSensitive (w, (item == 1) ? True : False);
}

void NewKennChangedCallback (Widget w, XtPointer tag, XtPointer data)
{
XmtChooserCallbackStruct *ch = (XmtChooserCallbackStruct *)data;
Widget wDisc = XmtNameToWidget (toplevel, "*rcl*lpDiscid");

	XtSetSensitive (wDisc, (ch->item == 1) ? True : False);
}

void UpdateKennChangedCallback (Widget w, XtPointer tag, XtPointer data)
{
XmtChooserCallbackStruct *ch = (XmtChooserCallbackStruct *)data;
Widget wDisc = XmtNameToWidget (toplevel, "*Updrcl*lpDiscid");

	XtSetSensitive (wDisc, (ch->item == 1) ? True : False);
}

/*
 * Dialogboxen zur Erfassung von einzelnen Cuts eines Tontraegers.
 */
void NewLPCut ()
{
int num = 0;

	XmtDisplayBusyCursor (toplevel);

	if (!XmtAskForInteger (toplevel, "DialogAskLPNum",
		"Record number:", &num, 1, 32767,
		"Enter the number of an already existing record label"))
	{
	   XmtDisplayDefaultCursor (toplevel);
	   return;
	}

	lpmaster.lpnum = (long)num;

	if (!rccWidget)
	   rccWidget = XmtBuildQueryDialog (toplevel, "rcc", 
		lpdchild_resource, XtNumber (lpdchild_resource), NULL);

	memset ((void *)&lpdchild, 0, sizeof (lpdchild_t));
	lpdchild.repeat = 1;
	lpdchild.snum = lpmaster.lpnum;

	if ((posCount = CountCuts (rccWidget, lpdchild.snum)) == -1)
	{
	   XmtDisplayDefaultCursor (toplevel);
	   return;
	}

	posCount++;
	lpdchild.spos = posCount;
	fromLps = False;
	Singer[0] = 0;

	XmtDialogDoSync (rccWidget, &lpdchild);
	XmtDisplayDefaultCursor (toplevel);
}

void NewLPCutDialog ()
{
Widget list = XmtNameToWidget (toplevel, "*lps*lpMaster");
int sel;
int row, col;

	XmtDisplayBusyCursor (qLpWidget);
	XbaeMatrixFirstSelectedCell (list, &row, &col);

	if (row < 0)
	{
	   XmtDisplayWarning (qLpWidget, "ErrorNoSelection", NULL);
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (!rccWidget)
	   rccWidget = XmtBuildQueryDialog (qLpWidget, "rcc", 
		lpdchild_resource, XtNumber (lpdchild_resource), NULL);

	memset ((void *)&lpdchild, 0, sizeof (lpdchild_t));
	lpdchild.repeat = 1;
	lpdchild.snum = lpmaster.lpnum;

	if ((posCount = CountCuts (rccWidget, lpdchild.snum)) == -1)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	posCount++;
	lpdchild.spos = posCount;
	fromLps = True;
	Singer[0] = 0;

	XmtDialogDoSync (rccWidget, &lpdchild);
	XmtDisplayDefaultCursor (qLpWidget);
}

int CountCuts (Widget w, long num)
{
EXEC SQL BEGIN DECLARE SECTION;
	long inum;
	int anzahl;
EXEC SQL END DECLARE SECTION;

	inum = num;
	EXEC SQL select count(*) into :anzahl from lpchild
			where lpchild.snum = :inum;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return -1;
	}

	return anzahl;
}

void AddLpCutsCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
int item, i, j;
String *row;
char hv0[128], hv1[128];
Widget list;

	if (fromLps)
	   list = XmtNameToWidget (toplevel, "*lps*lpCuts");
 
	XmtDisplayBusyCursor (rccWidget);
	XmtDialogGetDialogValues (rccWidget, &lpdchild);

	/*
	 * Zuerst pruefen ob alle Mussfelder versorgt wurden.
	 */
	if (!strlen (lpdchild.stitel) || !strlen (lpdchild.ssinger))
	{
	   XmtDisplayError (rccWidget, "MissingValue", NULL);
	   XmtDisplayDefaultCursor (rccWidget);
	   return;
	}

	/*
	 * Nun kann der Datensatz in die Datenbank eingefuegt werden.
	 */
	if (has_trans)
	   EXEC SQL begin work;

	lpchild.snum = lpdchild.snum;
	lpchild.spos = lpdchild.spos;
	strcpy (lpchild.stitel, lpdchild.stitel);
	strcpy (lpchild.ssinger, lpdchild.ssinger);
	strcpy (lpchild.szeit, lpdchild.szeit);

	EXEC SQL insert into lpchild values (:lpchild.snum,
		 :lpchild.spos, :lpchild.stitel, :lpchild.ssinger,
		 :lpchild.szeit);

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (rccWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (rccWidget);
	   return;
	}

	if (has_trans)
	   EXEC SQL commit work;

	if (fromLps)
	{
	   sprintf (hv0, "%ld", lpdchild.snum);
	   sprintf (hv1, "%d", lpdchild.spos);
	   row = (String *)XtMalloc (5 * sizeof (String));
	   row[0] = &hv0[0];
	   row[1] = &hv1[0];
	   row[2] = &lpdchild.stitel[0];
	   row[3] = &lpdchild.ssinger[0];
	   row[4] = &lpdchild.szeit[0];
	   XbaeMatrixAddRows (list, posCount-1, row, NULL, NULL, 1);
	   XtFree ((char *)row);
	   XbaeMatrixMakeCellVisible (list, posCount-1, 0);
	}

	posCount++;

	if (lpdchild.repeat & 1)
	   strcpy (Singer, lpdchild.ssinger);
	else
	   Singer[0] = 0;

	rccClearValuesCallback (w, tag, data);
	XmtDisplayDefaultCursor (rccWidget);
}

void rccClearValuesCallback (Widget w, XtPointer tag, XtPointer data)
{
int si;

	si = lpdchild.repeat;
	memset ((void *)&lpdchild, 0, sizeof (lpdchild_t));
	lpdchild.repeat = si;
	lpdchild.snum = lpmaster.lpnum;
	lpdchild.spos = posCount;
	strcpy (lpdchild.ssinger, Singer);
	XmtDialogSetDialogValues (rccWidget, &lpdchild);
}

/*
 * Folgende Funktion kann nur aus der Funktion "UpdateLpCallback"
 * aufgerufen werden. Die Funktion holt den selektierten Datensatz
 * aus der Datenbank und uebergibt die Daten einer Dialogbox.
 * Wenn Der User seine Aenderungen getaetigt hat, werden die Aenderungen
 * in die Datenbank eingetragen.
 */
void UpdateLpCuts (Widget list)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
	long inum;
	int ipos;
EXEC SQL END DECLARE SECTION;
/* lpchild_t lpchild; */
int i, row, col;
char num[11], hv0[128];
String sCellNum, sCellPos;
XmString cstr, cstrSep;
XmStringTable cstrt;
 
	XmtDisplayBusyCursor (qLpWidget);
	/*
	 * Als erstes muessen wir den zu aendernden Datensatz aus
	 * der Datenbank holen und in die Dialogbox eintragen.
	 */
	XbaeMatrixFirstSelectedCell (list, &row, &col);
	sCellNum = XbaeMatrixGetCell (list, row, 0);
	sCellPos = XbaeMatrixGetCell (list, row, 1);

	inum = atol (sCellNum);
	ipos = atoi (sCellPos);

	if (inum <= 0 || ipos <= 0)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	EXEC SQL select * into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit from lpchild where
			lpchild.snum = :inum and lpchild.spos = :ipos;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (!UpdrccWidget)
	   UpdrccWidget = XmtBuildQueryDialog (toplevel, "Updrcc", 
		lpchild_resource, XtNumber (lpchild_resource), NULL);

	rtrim (lpchild.stitel);
	rtrim (lpchild.ssinger);

	if (!XmtDialogDoSync (UpdrccWidget, &lpchild))
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	/*
	 * Nach dem der User hier etwas geaendert hat (oder auch nicht)
	 * muss es in die Datenbank eingetragen werden.
	 */
	if (has_trans)
	   EXEC SQL begin work;

	EXEC SQL update lpchild set stitel = :lpchild.stitel,
		 ssinger = :lpchild.ssinger, szeit = :lpchild.szeit
		 where snum = :inum and spos = :ipos;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);

	   if (has_trans)
	      EXEC SQL rollback work;

	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (has_trans)
	   EXEC SQL commit work;

	/*
	 * Um die Aenderung sichtbar zu machen, aendern wir noch die
	 * entsprechende Zeile in der Listbox.
	 */
	sprintf (hv0, "%ld", lpchild.snum);
	XbaeMatrixSetCell (list, row, 0, hv0);
	sprintf (hv0, "%d", lpchild.spos);
	XbaeMatrixSetCell (list, row, 1, hv0);
	XbaeMatrixSetCell (list, row, 2, lpchild.stitel);
	XbaeMatrixSetCell (list, row, 3, lpchild.ssinger);
	XbaeMatrixSetCell (list, row, 4, lpchild.szeit);
	XmtDisplayDefaultCursor (qLpWidget);
}

/*
 * Handling der Dialogbox zur Anzeige von Tontraegern
 */
void SearchLps ()
{
lpmask_t lpmask;
Widget list1, list2;

	XmtDisplayBusyCursor (toplevel);

	if (!qLpWidget)
	   qLpWidget = XmtBuildQueryDialog (toplevel, "lps", 
		lpmask_resource, XtNumber (lpmask_resource), NULL);
	else
	{
	   list1 = XmtNameToWidget (toplevel, "*lps*lpMaster");
	   list2 = XmtNameToWidget (toplevel, "*lps*lpCuts");
	   XbaeMatrixDeleteRows (list1, 0, XbaeMatrixNumRows (list1));
	   XbaeMatrixDeleteRows (list2, 0, XbaeMatrixNumRows (list2));
	   FirstFillCallback (qLpWidget, list1);
	}

	memset ((void *)&lpmask, 0, sizeof (lpmask_t));

	XmtDialogDoSync (qLpWidget, &lpmask);
	XmtDisplayDefaultCursor (toplevel);
}

void FirstFillCallback (Widget dialog, Widget list)
{
EXEC SQL BEGIN DECLARE SECTION;
	long anzahl, idate;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
char sdate[11], hv0[128];
String *row;
int i, j;
XmString cstr;

	XmtDisplayBusyCursor (dialog);
	EXEC SQL select count(*) into :anzahl from lpmaster;

	if (anzahl < 50L)
	   strcpy (sql_string, "select * from lpmaster order by lpnum");
	else
	   strcpy (sql_string, "select * from lpmaster where lptitel like 'A%' order by lpnum");

	EXEC SQL prepare lpid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (dialog);
	   XmtDisplayDefaultCursor (dialog);
	   return;
	}

	EXEC SQL declare lpcurs cursor for lpid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (dialog);
	   XmtDisplayDefaultCursor (dialog);
	   EXEC SQL free lpcurs;
	   return;
	}

	EXEC SQL open lpcurs;
	EXEC SQL fetch lpcurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;

	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (dialog);
	   XmtDisplayDefaultCursor (dialog);
	   EXEC SQL close lpcurs;
	   EXEC SQL free lpcurs;
	   return;
	}

	i = 0;
	XbaeMatrixDisableRedisplay (list);

	while (SQLCODE == 0)
	{
	   if (lpmaster.lpdate < 1000L)
	      sdate[0] = 0;
	   else
	      rfmtdate (lpmaster.lpdate, app_resources.datePattern, &sdate[0]);

	   sprintf (hv0, "%ld", lpmaster.lpnum);
	   row = (String *)XtMalloc (4 * sizeof (String));
	   row[0] = &hv0[0];
	   row[1] = &lpmaster.lptitel[0];
	   row[2] = &lpmaster.lpkenn[0];
	   row[3] = &sdate[0];
	   XbaeMatrixAddRows (list, i, row, NULL, NULL, 1);
	   XtFree ((char *)row);

	   i++;
	   EXEC SQL fetch lpcurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;
	}

	XbaeMatrixEnableRedisplay (list, True);
	EXEC SQL close lpcurs;
	EXEC SQL free lpcurs;
	XmtDisplayDefaultCursor (dialog);
}

void LPSearchValueChangedCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
char sdate[11];
lpmask_t lpmask;
int items, i, j;
String *row;
XmString cstr;
Widget list = XmtNameToWidget (toplevel, "*lps*lpMaster");

	XmtDisplayBusyCursor (qLpWidget);
	XmtDialogGetDialogValues (qLpWidget, &lpmask);

	if (lpmask.lpSearch[0] == 0)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (lpsSearch == lpsSINGER || lpsSearch == lpsSONG)
	{
	   lpsFillChild (lpmask.lpSearch);
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (lpsSearch == lpsTITEL)
	   sprintf (sql_string, "select * from lpmaster where lptitel matches '%s' order by lpnum",
		lpmask.lpSearch);
	else
	   sprintf (sql_string, "select * from lpmaster where lpnum = %d",
		atoi (lpmask.lpSearch));

	EXEC SQL prepare lpsid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	EXEC SQL declare lpscurs cursor for lpsid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   EXEC SQL free lpscurs;
	   return;
	}

	/*
	 * Bevor wir etwas in die ListBox einfuegen, muessen wir
	 * sicherstellen, dass die ListBox leer ist.
	 */
	items = XbaeMatrixNumRows (list);

        if (items > 0)
           XbaeMatrixDeleteRows (list, 0, items);

	EXEC SQL open lpscurs;
	EXEC SQL fetch lpscurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;

	if (SQLCODE == SQLNOTFOUND)
	{
	   XmtDisplayError (qLpWidget, "iNotFound", NULL);
	   EXEC SQL close lpsccurs;
	   EXEC SQL free lpsccurs;
	   return;
	}
	   
	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   EXEC SQL close lpscurs;
	   EXEC SQL free lpscurs;
	   return;
	}

	i = 0;
	XbaeMatrixDisableRedisplay (list);

	while (SQLCODE == 0)
	{
	   if (lpmaster.lpdate < 1000L)
	      sdate[0] = 0;
	   else
	      rfmtdate (lpmaster.lpdate, app_resources.datePattern, &sdate[0]);

	   sprintf (stmaster, "%ld", lpmaster.lpnum);
	   row = (String *)XtMalloc (4 * sizeof (String));
	   row[0] = &stmaster[0];
	   row[1] = &lpmaster.lptitel[0];
	   row[2] = &lpmaster.lpkenn[0];
	   row[3] = &sdate[0];
	   XbaeMatrixAddRows (list, i, row, NULL, NULL, 1);
	   XtFree ((char *)row);

	   i++;
	   EXEC SQL fetch lpscurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;
	}

	XbaeMatrixEnableRedisplay (list, True);
	EXEC SQL close lpscurs;
	EXEC SQL free lpscurs;
	XmtDisplayDefaultCursor (qLpWidget);
}

void MasterListCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
XbaeMatrixEnterCellCallbackStruct *ec = (XbaeMatrixEnterCellCallbackStruct *)data;
char num[10], pos[10];
int items, i;
String cstr, *zeile;
Widget list = XmtNameToWidget (toplevel, "*lps*lpCuts");

	ec->doit = False;
	XbaeMatrixDeselectAll (w);
	XbaeMatrixSelectRow (w, ec->row);
	XbaeMatrixUnhighlightCell (w, ec->row, ec->column);
	cstr = XbaeMatrixGetCell (w, ec->row, 0);
	lpmaster.lpnum = atol (cstr);

	if (lpmaster.lpnum <= 0L)
	{
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	sprintf (sql_string, "select * from lpchild where lpchild.snum = %s order by snum,spos", cstr);
	
	EXEC SQL prepare lpcid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	EXEC SQL declare lpccurs cursor for lpcid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   EXEC SQL free lpccurs;
	   return;
	}

	/*
	 * Bevor wir etwas in die ListBox einfuegen, muessen wir
	 * sicherstellen, dass die ListBox leer ist.
	 */
	items = XbaeMatrixNumRows (list);

        if (items > 0)
           XbaeMatrixDeleteRows (list, 0, items);

	EXEC SQL open lpccurs;
	EXEC SQL fetch lpccurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;

	if (SQLCODE == 100)	/* wurden Daten gefunden? */
	{
	   EXEC SQL close lpccurs;
	   EXEC SQL free lpccurs;
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   XmtDisplayDefaultCursor (qLpWidget);
	   EXEC SQL close lpccurs;
	   EXEC SQL free lpccurs;
	   return;
	}

	i = 0;

	while (SQLCODE == 0)
	{
	   sprintf (num, "%ld", lpchild.snum);
	   sprintf (pos, "%d", lpchild.spos);
	   zeile = (String *)XtMalloc (5 * sizeof (String));
	   zeile[0] = &num[0];
	   zeile[1] = &pos[0];
	   zeile[2] = &lpchild.stitel[0];
	   zeile[3] = &lpchild.ssinger[0];
	   zeile[4] = &lpchild.szeit[0];
	   XbaeMatrixAddRows (list, i, zeile, NULL, NULL, 1);
	   XtFree ((char *)zeile);

	   i++;
	   EXEC SQL fetch lpccurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;
	}

	EXEC SQL close lpccurs;
	EXEC SQL free lpccurs;
	XmtDisplayDefaultCursor (qLpWidget);
}

void lpsPartChooserCallback (Widget w, XtPointer tag, XtPointer data)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
char sdate[11], hv0[128], *p;
lpmask_t lpmask;
int items, i;
String *cstr;
Widget list = XmtNameToWidget (toplevel, "*lps*lpMaster");
XmtChooserCallbackStruct *Chooser = (XmtChooserCallbackStruct *)data;
Widget wSearch = XmtNameToWidget (toplevel, "*lps*lpSearch");

	if (lpsSearch == lpsNUMBER)
	   return;

	p = *(String *)Chooser->valuep;
	XmtInputFieldSetString (wSearch, p);

	if (lpsSearch == lpsSINGER || lpsSearch == lpsSONG)
	{
	   lpsFillChild (p);
	   return;
	}

	sprintf (sql_string,
		"select * from lpmaster where lptitel matches '%s' order by lpnum", p);

	EXEC SQL prepare lpscid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return;
	}

	EXEC SQL declare lpsccurs cursor for lpscid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   EXEC SQL free lpsccurs;
	   return;
	}

	/*
	 * Bevor wir etwas in die ListBox einfuegen, muessen wir
	 * sicherstellen, dass die ListBox leer ist.
	 */
	items = XbaeMatrixNumRows (list);

        if (items > 0)
           XbaeMatrixDeleteRows (list, 0, items);

	EXEC SQL open lpsccurs;
	EXEC SQL fetch lpsccurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;

	if (SQLCODE == SQLNOTFOUND)
	{
	   XmtDisplayError (w, "iNotFound", NULL);
	   EXEC SQL close lpsccurs;
	   EXEC SQL free lpsccurs;
	   return;
	}
	   
	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (w);
	   EXEC SQL close lpsccurs;
	   EXEC SQL free lpsccurs;
	   return;
	}

	i = 0;

	while (SQLCODE == 0)
	{
	   if (lpmaster.lpdate < 1000L)
	      sdate[0] = 0;
	   else
	      rfmtdate (lpmaster.lpdate, app_resources.datePattern, &sdate[0]);

	   sprintf (hv0, "%ld", lpmaster.lpnum);
	   cstr = (String *)XtMalloc (4 * sizeof (String));
	   cstr[0] = &hv0[0];
	   cstr[1] = lpmaster.lptitel;
	   cstr[2] = lpmaster.lpkenn;
	   cstr[3] = sdate;
	   XbaeMatrixAddRows (list, i, cstr, NULL, NULL, 1);
	   XtFree ((char *)cstr);

	   i++;
	   EXEC SQL fetch lpsccurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate;
	}

	EXEC SQL close lpsccurs;
	EXEC SQL free lpsccurs;
}

void lpsSearchTypeCallback (Widget w, XtPointer tag, XtPointer data)
{
XmtChooserCallbackStruct *Chooser = (XmtChooserCallbackStruct *)data;

	lpsSearch = Chooser->item;
}

/*
 * Folgende Routine wird dann aufgerufen, wenn als Suchbegriff der
 * Interpret gewhlt wurde. In diesem Fall erscheinen dann alle
 * Suchergebnisse in der Child-Listbox. Die Master-Listbox wird
 * geleert um die Verwirrung geringer zu halten.
 */
void lpsFillChild (char *str)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string[256];
EXEC SQL END DECLARE SECTION;
char num[10], pos[10];
int items, i;
String *cstr;
Widget mlist = XmtNameToWidget (toplevel, "*lps*lpMaster");
Widget list = XmtNameToWidget (toplevel, "*lps*lpCuts");

	/*
	 * Bevor wir etwas in die Child-ListBox einfuegen, muessen wir
	 * sicherstellen, dass die Master-ListBox leer ist.
	 */
	items = XbaeMatrixNumRows (mlist);

        if (items > 0)
           XbaeMatrixDeleteRows (mlist, 0, items);

	if (lpsSearch == lpsSINGER)
	   sprintf (sql_string, "select * from lpchild where lpchild.ssinger matches '%s' order by ssinger,snum,spos", str);
	else
	   sprintf (sql_string, "select * from lpchild where lpchild.stitel matches '%s' order by stitel,snum,spos", str);
	
	EXEC SQL prepare chiid from :sql_string;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   return;
	}

	EXEC SQL declare chicurs cursor for chiid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   EXEC SQL free chicurs;
	   return;
	}

	/*
	 * Bevor wir etwas in die ListBox einfuegen, muessen wir
	 * sicherstellen, dass die ListBox leer ist.
	 */
	items = XbaeMatrixNumRows (list);

        if (items > 0)
           XbaeMatrixDeleteRows (list, 0, items);

	EXEC SQL open chicurs;
	EXEC SQL fetch chicurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;

	if (SQLCODE == 100)	/* wurden Daten gefunden? */
	{
	   EXEC SQL close chicurs;
	   EXEC SQL free chicurs;
	   XmtDisplayDefaultCursor (qLpWidget);
	   return;
	}

	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (qLpWidget);
	   EXEC SQL close chicurs;
	   EXEC SQL free chicurs;
	   return;
	}

	i = 0;

	while (SQLCODE == 0)
	{
	   sprintf (num, "%ld", lpchild.snum);
	   sprintf (pos, "%d", lpchild.spos);
	   cstr = (String *)XtMalloc (5 * sizeof (String));
	   cstr[0] = &num[0];
	   cstr[1] = &pos[0];
	   cstr[2] = lpchild.stitel;
	   cstr[3] = lpchild.ssinger;
	   cstr[4] = lpchild.szeit;
	   XbaeMatrixAddRows (list, i, cstr, NULL, NULL, 1);
	   XtFree ((char *)cstr);

	   i++;
	   EXEC SQL fetch chicurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;
	}

	EXEC SQL close chicurs;
	EXEC SQL free chicurs;
}

/*
 * Folgende Routine nimmt als Parameter eine Plattennummer und gibt
 * dann den zugehoerigen Plattenlabel in der Mastertabelle aus. Die
 * einzelnen Cuts werden in der Childtabelle ausgegeben. Die Routine
 * wird aufgerufen, nach dem eine CD automatisch aus dem CDDB
 * Database eingefgt wurde.
 */
void lpsListLp (Widget w, long num)
{
EXEC SQL BEGIN DECLARE SECTION;
	char sql_string1[256];
	char sql_string2[256];
EXEC SQL END DECLARE SECTION;
char snum[10], pos[10];
int items, i;
String *cstr;
Widget mlist = XmtNameToWidget (toplevel, "*lps*lpMaster");
Widget list = XmtNameToWidget (toplevel, "*lps*lpCuts");

	/*
	 * Bevor wir etwas in die ListBoxen einfuegen, muessen wir
	 * sicherstellen, dass die ListBoxen leer sind.
	 */
	items = XbaeMatrixNumRows (mlist);

        if (items > 0)
           XbaeMatrixDeleteRows (mlist, 0, items);

	items = XbaeMatrixNumRows (list);

	if (items > 0)
	   XbaeMatrixDeleteRows (list, 0, items);

	sprintf (sql_string1, "select * from lpmaster where lpmaster.lpnum = %ld", num);
	sprintf (sql_string2, "select * from lpchild where lpchild.snum = %ld", num);
	
	EXEC SQL prepare infoMid from :sql_string1;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return;
	}

	EXEC SQL prepare infoCid from :sql_string2;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return;
	}

	EXEC SQL declare infoMcurs cursor for infoMid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return;
	}

	EXEC SQL open infoMcurs;

	EXEC SQL fetch infoMcurs into :lpmaster.lpnum, :lpmaster.lptitel,
			:lpmaster.lpkenn, :lpmaster.lpdate,
			:lpmaster.lpCategory, :lpmaster.lpDiscid;

	if (SQLCODE == 100)
	{
	   EXEC SQL close infoMcurs;
	   EXEC SQL free ibfoMcurs;
	   return;
	}

	sprintf (snum, "%ld", lpmaster.lpnum);

	if (lpmaster.lpdate < 1000L)
	   pos[0] = 0;
	else
	   rfmtdate (lpmaster.lpdate, app_resources.datePattern, &pos[0]);

	cstr = (String *)XtMalloc (4 * sizeof (String));
	cstr[0] = &snum[0];
	cstr[1] = lpmaster.lptitel;
	cstr[2] = lpmaster.lpkenn;
	cstr[3] = &pos[0];
	XbaeMatrixAddRows (mlist, 0, cstr, NULL, NULL, 1);
	XtFree ((char *)cstr);
	EXEC SQL close infoMcurs;
	EXEC SQL free infoMcurs;

	EXEC SQL declare infoCcurs cursor for infoCid;

	if (SQLCODE != 0 || CheckSqlError ())
	{
	   DisplaySqlError (w);
	   return;
	}

	EXEC SQL open infoCcurs;

	EXEC SQL fetch infoCcurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;

	if (SQLCODE == 100)	/* wurden Daten gefunden? */
	{
	   EXEC SQL close infoCcurs;
	   EXEC SQL free infoCcurs;
	   return;
	}

	if (SQLCODE != 0 && CheckSqlError ())
	{
	   DisplaySqlError (w);
	   EXEC SQL close infoCcurs;
	   EXEC SQL free infoCcurs;
	   return;
	}

	i = 0;

	while (SQLCODE == 0)
	{
	   sprintf (snum, "%ld", lpchild.snum);
	   sprintf (pos, "%d", lpchild.spos);
	   cstr = (String *)XtMalloc (5 * sizeof (String));
	   cstr[0] = &snum[0];
	   cstr[1] = &pos[0];
	   cstr[2] = lpchild.stitel;
	   cstr[3] = lpchild.ssinger;
	   cstr[4] = lpchild.szeit;
	   XbaeMatrixAddRows (list, i, cstr, NULL, NULL, 1);
	   XtFree ((char *)cstr);

	   i++;
	   EXEC SQL fetch infoCcurs into :lpchild.snum, :lpchild.spos,
			:lpchild.stitel, :lpchild.ssinger,
			:lpchild.szeit;
	}

	EXEC SQL close infoCcurs;
	EXEC SQL free infoCcurs;
}
